from typing import Dict, Tuple
import re

from ..domain import InputPluginAbstract, DeviceInfo

# 类思科的情况
# DEVICE_NAME>show version
# device_name(config)# sh version
similar_cisco_reg = r'^(?P<device_name>[\w\-_]+)(?:\(.*?\))?[#|>]\s*(?P<cmd>sh(?:o(?:w)?)?\s+.*)$'

# 类华为的情况
# <device>display version
# [device-Eth0] dis version
simialr_huawei_reg = r'[\<|\[](?P<device_name>[\w_]+)(?:\-(?:.+?))?[\]|>]\s*(?P<cmd>dis(?:p(?:l(?:a(?:y)?)?)?)?\s+.*)$'


class InputPluginWithConsole(InputPluginAbstract):
    """通过Console获取命令的输出"""

    def main(self, file_path: str, stream: str) -> Tuple[Dict[str, str], DeviceInfo]:
        device_name = ''
        command = ''
        cmd_dict = {}
        content = []
        for line in stream.splitlines():
            match = re.match(similar_cisco_reg, line) or \
                re.match(simialr_huawei_reg, line)  # 判断是否为命令的行
            if match:
                if not device_name:
                    device_name = match.group('device_name')

                if content and command:  # 如果有内容，且有命令，则保存
                    cmd_dict[command] = '\n'.join(content)

                command = match.group('cmd').strip()
                content.clear()
                continue

            content.append(line)

        if content and command:
            cmd_dict[command] = '\n'.join(content)

        return cmd_dict, DeviceInfo(device_name, '')



class Error(Exception):
    ...


class Continue(Exception):
    """用于控制循环"""


class TemplateError(Error):
    """ntc-template 错误"""


class PluginError(Error):
    ...


class NotPluginError(Error):
    ...


class AnalysisPluginError(Error):
    ...


class AnalysisLevelError(AnalysisPluginError):
    ...


class CommandNotFoundError(AnalysisPluginError):
    ...


class AnalysisTemplateNameError(AnalysisPluginError):
    """模板的名称不存在"""


class AnalysisVendorNotSupport(AnalysisPluginError):
    """分析模块暂不支持该厂商"""


class NtcTemplateNotDefined(AnalysisPluginError):
    """模板名称不存在"""



__version__ = '1.2.0'

from .api import NetInspect
from .base_info import (
    get_base_info,
    EachVendorDeviceInfo,
    BaseInfo,
    AnalysisInfo
)
from .domain import (
    InputPluginAbstract,
    OutputPluginAbstract,
    ParsePluginAbstract,
    DeviceInfo,
    DeviceList,
    Device,
    Cmd
)
from .exception import (
    NotPluginError,
    PluginError,
    TemplateError
)



from __future__ import annotations
import os
import re
from typing import TYPE_CHECKING, Dict, List
from ntc_templates.parse import parse_output, __file__ as model_file

from ..domain import ParsePluginAbstract
from .. import exception
from ..func import reg_extend
from ..data import pyoption

if TYPE_CHECKING:
    from ..domain import Cmd


class ParsePluginWithNtcTemplates(ParsePluginAbstract):
    """
    使用ntc-templates解析命令
    仓库地址：
        https://github.com/Elinpf/ntc-templates
    """

    def __init__(self):
        self.index_commands: Dict[str, List[str]] = self._get_index_commands()

    def _get_index_commands(self) -> Dict[str, List[str]]:
        """将ntc-templates中的index文件提取出来，
        并将命令按照平台分类"""

        ntc_template_path = os.path.dirname(model_file)
        index_file = os.path.join(ntc_template_path, 'templates', 'index')
        commands = {}
        with open(index_file, 'r') as f:
            for line in f.readlines():
                if line == '\n':
                    continue
                elif line.startswith('#'):
                    continue
                elif "Template, Hostname, Platform, Command" in line:
                    continue

                e = line.split(',')
                if len(e) != 4:  # 判断是有效行
                    continue

                platform = e[2].strip()
                if platform not in commands:
                    commands[platform] = []

                commands[platform].append(e[-1].strip())

        return commands

    def main(self, cmd: Cmd, platform: str) -> Dict[str, str]:
        """对命令进行解析
        :param cmd: 命令对象
        :param platform: 命令所属平台, e.g. huawei_os"""
        command = cmd.command
        if platform not in self.index_commands:
            if pyoption.verbose_level >= 3:
                raise exception.TemplateError(
                    f'platform:{platform!r} not support')
            raise exception.Continue

        platform_commands_reg = self.index_commands[platform]
        match_command = ''
        for cmd_reg in platform_commands_reg:
            if re.match(f'^{reg_extend(cmd_reg)}$', command):
                match_command = re.sub(r'[\[|\]]', '', cmd_reg)
                break

        if match_command == '':
            if pyoption.verbose_level >= 3:
                raise exception.TemplateError(
                    f'platform:{platform!r} cmd:{command!r} command not support')
            raise exception.Continue

        try:
            res = parse_output(platform=platform,
                               command=match_command, data=cmd.content)
        except Exception as e:
            raise exception.TemplateError(
                f'platform: {platform!r} cmd: {command!r} {str(e)}')

        if not res:  # 如果没有解析到结果，则抛出异常提示
            raise exception.TemplateError(
                f'platform:{platform!r} cmd:{command!r} no parse result')
        return res



from __future__ import annotations

from .plugins import autoload_plugin, PluginRepository
from .domain import (
    InputPluginAbstract,
    OutputPluginAbstract,
    ParsePluginAbstract
)

from .analysis_plugin import AnalysisPluginAbc


def bootstrap() -> PluginRepository:
    """加载启动项"""
    autoload_plugin()

    plugin_abc = (InputPluginAbstract, OutputPluginAbstract,
                  ParsePluginAbstract, AnalysisPluginAbc)
    plugins = []
    for plugin_abstract in plugin_abc:
        temp = plugin_abstract.__subclasses__()
        plugins.append({t.__name__: t for t in temp})

    return PluginRepository(*plugins)



import re
import sys

from .data import pyoption
from .logger import log


def reg_extend(reg: str) -> str:
    """扩展正则表达式写法，支持简单的逐字匹配
    e.g
    r'sh[[ow]] ver[[sion]]'
    将变成
    'sh(o(w)?)? ver(s(i(o(n)?)?)?)?'
    """
    def _completion(match):
        word = str(match.group())[2:-2]
        return '(' + ('(').join(word) + ')?' * len(word)

    return re.sub(r'(\[\[.+?\]\])', _completion, reg)


def get_command_from_textfsm(vendor_platform: str, template: str) -> str:
    """从模板文件名中获得命令
    :param vendor_platform: 平台名称
    :param template: 模板文件名

    :return: 命令

    >>> self.get_command('huawei_os', 'huawei_os_display_interface_status.textfsm')
    'display interface status'
    """

    template = template.replace(vendor_platform + '_', '')
    template = template.replace('.textfsm', '')
    return template.replace('_', ' ')


def pascal_case_to_snake_case(camel_case: str) -> str:
    """大驼峰（帕斯卡）转蛇形

    >>> pascal_case_to_snake_case('HuaweiVrpDisplayVersion')
    'huawei_vrp_display_version'
    """
    snake_case = re.sub(r"(?P<key>[A-Z])", r"_\g<key>", camel_case)
    return snake_case.lower().strip('_')


def snake_case_to_pascal_case(snake_case: str) -> str:
    """蛇形转大驼峰（帕斯卡）

    >>> snake_case_to_pascal_case('huawei_vrp_display_version')
    'HuaweiVrpDisplayVersion'
    """
    words = snake_case.split('_')
    return ''.join(word.title() for word in words)


def clamp_number(num: int, min_num: int, max_num: int) -> int:
    """
    将数字限制在指定范围内

    Args:
        num: 数字
        min_num: 最小值
        max_num: 最大值 

    Returns:
        int: 限制后的数字
    """
    return max(min(num, max_num), min_num)


def match(string: str, pattern: str) -> bool:
    """匹配字符串，区分大小写"""
    return bool(re.match(pattern, string))


def match_lower(string: str, pattern: str) -> bool:
    """匹配字符串，不区分大小写"""
    return bool(re.match(pattern, string.lower()))


def print_log(string: str, verbose: int = 0) -> None:
    """
    根据级别打印日志

    Args:
        string: 日志内容
        verbose: 日志级别

    `verbose`级别可以通过`verbose()`方法设置，总共0~3
    - 0: 日志关闭
    - 1: 提供Output模块的日志和Parse模块的日志
    - 2: 追加提供Analysis模块的日志
    - 3: 追加提供Parse模块不支持命令的信息日志和命令为无效的信息
    """
    verbose = clamp_number(verbose, 1, 3)
    if pyoption.verbose_level >= verbose:
        log.debug(string)


class SkipWithBlock(Exception):
    pass


class Singleton(object):
    """单例类继承"""
    _instance = None

    def __new__(class_, *args, **kwargs):
        if not isinstance(class_._instance, class_):
            class_._instance = object.__new__(class_, *args, **kwargs)
        return class_._instance


class NoneSkip(Singleton):
    """
    简单模仿None类，但是可以用来作为-with-中的跳过
    使用bool()来判断是否为空
    """

    def __init__(self):
        self.none = None

    def __enter__(self):
        if sys.gettrace():
            return self

        # NOTE 存在一定的问题，当程序处于调试状态的时候，调试进程会被破坏：
        # https://pydev.blogspot.com/2007/06/why-cant-pydev-debugger-work-with.html
        sys.settrace(lambda *args, **keys: None)
        frame = sys._getframe(1)
        frame.f_trace = self.trace

    def trace(self, frame, event, arg):
        raise SkipWithBlock()

    def __bool__(self):
        return False

    def __exit__(self, type, value, traceback):
        if type is None:
            return  # No exception
        if issubclass(type, SkipWithBlock):
            return True  # Suppress special SkipWithBlock exception

    def __eq__(self, other):
        return self.none == other

    def __ne__(self, other):
        return self.none != other

    def __getattr__(self, __name: str):
        # NOTE 当处于Debug状态的时候，调用不存在的属性时，就会触发这个，跳过-with-后面的内容
        raise SkipWithBlock()



import logging
from rich.logging import RichHandler

from .data import pystr, pyoption

LOG_KEYWORDS = [pystr.analysis_warning_prefix, pystr.parse_waning_prefix]

FORMAT = pystr.logger_format
logging.basicConfig(
    level=pyoption.log_level,
    format=FORMAT,
    datefmt="[%X]",
    handlers=[RichHandler(rich_tracebacks=True, tracebacks_show_locals=True,
                          show_path=False, show_time=False, keywords=LOG_KEYWORDS)])

log = logging.getLogger(pystr.software)



from __future__ import annotations
import os
from typing import Dict, List, Tuple

from .domain import PluginManagerAbc, DeviceInfo
from .data import pyoption


def _textfsm_reslut_to_dict(header: list, reslut: list) -> List[Dict[str, str]]:
    """将 TextFSM 的结果与header结合转化为dict"""
    objs = []
    for row in reslut:
        temp_dict = {}
        for index, element in enumerate(row):
            temp_dict[header[index].lower()] = element
        objs.append(temp_dict)

    return objs


class PluginManager(PluginManagerAbc):

    def input_dir(self, dir_path: str, expend: str | List = None) -> List[Tuple[Dict[str, str], DeviceInfo]]:
        """对目录中的文件进行设备输入"""
        devices = []

        if expend is None:
            expend = pyoption.log_file_expend
        elif expend is str:
            expend = [expend]

        for root, dirs, files in os.walk(dir_path):
            for file in files:
                if os.path.splitext(file)[-1] in expend:  # 判断文件后缀
                    file_path = os.path.join(root, file)
                    devices.append(self.input(file_path))

        return devices



from rich import print
from rich.table import Table
from ..domain import OutputPluginAbstract


class OutputPluginWithDeviceList(OutputPluginAbstract):
    """
    输出设备列表信息
    """

    def main(self):
        table = Table(title='设备列表')
        columns = ['设备名称', '厂商', '型号', 'IP']
        for col in columns:
            table.add_column(col)

        for device in self.args.devices:
            row = [
                device.info.hostname,
                device.info.vendor,
                device.info.model,
                device.info.ip
            ]
            table.add_row(*row)

        print(table)



from __future__ import annotations

import os
from typing import TYPE_CHECKING, Dict, List, Optional, Type

from .base_info import get_base_info
from .bootstrap import bootstrap
from .data import pyoption, pystr
from .domain import Cluster
from .func import clamp_number
from .logger import log
from .plugin_manager import PluginManager

if TYPE_CHECKING:
    from .base_info import EachVendorDeviceInfo, BaseInfo
    from .domain import (Device, InputPluginAbstract, OutputPluginAbstract,
                         ParsePluginAbstract, PluginAbstract)


class NetInspect:

    def __init__(self):
        self._plugins = bootstrap()
        self._plugin_manager = PluginManager()
        self.cluster = Cluster()

        self._plugin_manager.parse_plugin = self._plugins.get_parse_plugin(
            pystr.default_parse_plugin)
        self._plugin_manager.analysis_plugin = self._plugins.get_analysis_plugin_list()
        self.cluster._plugin_manager = self._plugin_manager

    def set_log_level(self, level: int | str):
        """设置日志等级"""
        log.setLevel(level)

    def get_all_plugins(self) -> Dict[str, Dict[str, Type[PluginAbstract]]]:
        """获取所有插件"""
        return {'input_plugins': self.get_input_plugins(),
                'output_plugins': self.get_output_plugins(),
                'parse_plugins': self.get_parse_plugins(),
                'analysis_plugins': self.get_analysis_plugins()}

    def get_input_plugins(self) -> Dict[str, Type[PluginAbstract]]:
        """取得所有的输入插件"""
        return self._plugins.input_plugins

    def get_output_plugins(self) -> Dict[str, Type[PluginAbstract]]:
        """取得所有的输出插件"""
        return self._plugins.output_plugins

    def get_parse_plugins(self) -> Dict[str, Type[PluginAbstract]]:
        """取得所有的解析插件"""
        return self._plugins.parse_plugins

    def get_analysis_plugins(self) -> Dict[str, Type[PluginAbstract]]:
        """取得所有的分析插件"""
        return self._plugins.analysis_plugins

    def set_input_plugin(self, plugin_cls: Type[InputPluginAbstract] | str):
        """设置输入插件
        :param plugin_cls: 插件类或者插件名称"""
        if type(plugin_cls) == str:
            plugin_cls = self._plugins.get_input_plugin(plugin_cls)
        self._plugin_manager.input_plugin = plugin_cls

    def set_output_plugin(self, plugin_cls: Type[OutputPluginAbstract] | str):
        """设置输出插件
        :param plugin_cls: 插件类或者插件名称"""
        if type(plugin_cls) == str:
            plugin_cls = self._plugins.get_output_plugin(plugin_cls)
        self._plugin_manager.output_plugin = plugin_cls

    def set_parse_plugin(self, plugin_cls: Type[ParsePluginAbstract] | str):
        """设置解析插件
        :param plugin_cls: 插件类或者插件名称"""
        if type(plugin_cls) == str:
            plugin_cls = self._plugins.get_parse_plugin(plugin_cls)
        self._plugin_manager.parse_plugin = plugin_cls

    def set_plugins(self,
                    input_plugin: Type[InputPluginAbstract] | str,
                    output_plugin: Type[OutputPluginAbstract] | str,
                    parse_plugin: Optional[Type[ParsePluginAbstract]] | str = None):
        """设置插件
        :param input_plugin: 输入插件
        :param output_plugin: 输出插件
        :param parse_plugin: 解析插件"""
        self.set_input_plugin(input_plugin)
        self.set_output_plugin(output_plugin)
        if parse_plugin:
            self.set_parse_plugin(parse_plugin)

    def run_input(self, path: str) -> Cluster:
        """运行输入插件
        :param path: 文件或者目录路径"""

        if os.path.isfile(path):
            self.cluster.input(path)
        elif os.path.isdir(path):
            self.cluster.input_dir(path)
        else:
            raise ValueError('path must be a file or directory')

        return self.cluster

    def run_parse(self) -> Cluster:
        """运行解析插件"""
        self.cluster.parse()
        return self.cluster

    def run_analysis(self) -> Cluster:
        """运行分析插件"""
        self.cluster.analysis()
        return self.cluster

    def run_output(self, file_path: str = '', params: Dict[str, str] = {}):
        """运行输出插件"""
        self.cluster.output(file_path, params)

    def run(self, path: str, output_file_path: str = '', output_plugin_params: Dict[str, str] = {}) -> Cluster:
        """运行输入解析输出插件
        :param path: 文件或者目录路径
        :param output_file_path: 输出文件路径
        :param output_plugin_params: 输出插件参数

        :return Cluster: 设备列表"""
        self.run_input(path)
        self.run_parse()
        self.run_analysis()

        if self._plugin_manager.output_plugin:  # 当有output插件的时候执行
            self.run_output(output_file_path, output_plugin_params)

        return self.cluster

    def search(self, device_name: str) -> List[Device]:
        """搜索设备
        :param device_name: 设备名称
        :return: 设备列表"""
        return self.cluster.search(device_name)

    def verbose(self, verbose: int):
        """设置输出等级

        Args:
            verbose: 输出等级 0~3
        """
        verbose = clamp_number(verbose, 0, 3)
        if verbose >= 1:
            self.set_log_level('DEBUG')
        else:
            self.set_log_level('INFO')
        pyoption.verbose_level = verbose

    def get_base_info(self) -> List[BaseInfo]:
        """获取所有设备的基本信息"""
        ret = []
        for device in self.cluster.devices:
            ret.append(device.info)

        return ret

    def set_base_info_handler(self, handler: Type[EachVendorDeviceInfo]):
        """设置设备基本信息处理器
        Args:
         - handler 设备基本信息处理器"""
        self.cluster.base_info_handler = handler()



import os
from . import __file__ as module_file


class PyStr:
    def __init__(self):
        self.software = 'net_inspect'
        self.logger_format = "%(message)s"
        # self.logger_format = "%(asctime)-15s - %(levelname)s - %(message)s"
        self.default_parse_plugin = 'ntc_templates'
        self.analysis_warning_prefix = 'AnalysisWarning --'
        self.parse_waning_prefix = 'ParseWarning --'


class PyOption:
    def __init__(self):
        self.log_file_expend = ['.txt', '.log', '.diag']
        self.log_level = 'INFO'
        self.verbose_level = 0


class PyPath:
    def __init__(self):
        self.root_path = os.path.dirname(module_file)
        self.project_path = os.path.dirname(self.root_path)


pystr = PyStr()
pyoption = PyOption()
pypath = PyPath()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['net_inspect', 'net_inspect.plugins', 'net_inspect.third_party']

package_data = \
{'': ['*']}

install_requires = \
['ntc_templates_elinpf>=3.2.0,<4.0.0',
 'openpyxl>=3.0.0,<4.0.0',
 'python-Levenshtein-wheels>=0.13.2,<0.14.0',
 'rich>=12.4.1,<13.0.0',
 'rich_typer>=0.1.5,<0.2.0',
 'textfsm>=1.1.2,<2.0.0']

entry_points = \
{'console_scripts': ['net_inspect = net_inspect.cli:app']}

setup_kwargs = {
    'name': 'net-inspect',
    'version': '1.2.0',
    'description': '基于已收集的网络设备信息进行的结构化数据分析框架',
    'long_description': '# net_inspect 介绍\n\n`net_inspect`是网络设备数据结构化分析框架\n\n\n## 安装方法\n\n### PyPI\n\n```bash\npip install net_inspect\n```\n\n### Poetry\n\n```bash\ngit clone https://github.com/Elinpf/net_inspect\ncd net_inspect\npoetry install\n```\n\n## 插件介绍\n\n`net_inspect`采用了插件模块的框架，分别是`Input`, `Parse`, `Analysis`, `Output` 模块。\n\n通过编写模块插件，实现快速定制的能力。\n\n### InputPlugin\n\nInput插件的功能是将**已经获得的设备检查命令的日志**转化为命令与命令输出的对应关系。如果是通过直接对设备进行操作获取的日志，可以使用`console`这个插件进行处理。如果是用的第三方平台进行的自动化收集，那么就需要自行编写Input插件实现命令与输出的对应。\n\n### ParsePlugin\n\nParse插件的功能是将每条命令的输出进行解析并且结构化。提取出关键信息并以`List[dict]`的方式进行存储。\n\n现有的`Parse`解析模块使用的是[ntc-templates-elinpf](https://github.com/Elinpf/ntc-templates)这个库，是`ntc-templates`库的分支，由于主仓更新频率很慢且不会增加国内常用的设备厂家，所以我Fork后进行了修改。\n\n\n### AnalysisPlugin\n\nAnalysis插件的功能是将解析的信息进行分析，对分析的内容进行告警分级。例如电源是否异常。这个工作是在分析模块中进行的。\n\n### OutputPlugin\n\n输出模块可能是最需要自定义编写的地方。将解析和分析的结果按照自己想要的格式展现出来。\n\n## 使用方法\n\n`net_inspect`有三种使用方式\n\n1. 作为三方库提供API\n2. CLI命令行操作\n3. 本地Web界面操作 (TODO)\n\n\n### 使用库\n\n示例:\n\n```python\nfrom net_inspect import NetInspect, OutputPluginAbstract, PluginError\nfrom rich.table import Table\nfrom rich.console import Console\n\n\nclass Output(OutputPluginAbstract):\n    def main(self):\n        if not self.args.output_params.get(\'company\'):\n            raise PluginError(\'name or age is missing\')\n\n        console = Console()\n\n        table = Table(title=self.args.output_params.get(\n            \'company\'), show_lines=False)\n        table.add_column(\'name\', justify=\'center\')\n        table.add_column(\'ip\', justify=\'center\')\n        table.add_column(\'model\', justify=\'center\')\n        table.add_column(\'version\', justify=\'center\')\n        table.add_column(\'power\', justify=\'center\')\n\n        for device in self.args.devices:\n            if device.vendor.PLATFORM == \'huawei_vrp\':\n                data = [device.info.name, device.info.ip]\n                ps = device.parse_result(\'display version\')\n                data.append(ps[0].get(\'model\'))\n                data.append(ps[0].get(\'vrp_version\'))\n                power_analysis = device.analysis_result.get(\'Power Status\')\n                power_desc = []\n                for alarm in power_analysis:\n                    if alarm.above_focus:\n                        power_desc.append(alarm.message)\n                data.append(\'\\n\'.join(power_desc) if power_desc else \'Normal\')\n\n                table.add_row(*data)\n                table.row_styles = [\'green\']\n\n        console.print(table)\n\n\nnet = NetInspect()\n# net.set_log_level(\'DEBUG\')\nnet.set_plugins(\'smartone\', Output)\ncluster = net.run(\'log_files\', output_plugin_params={\'company\': \'Company Name\'})\n```\n\n## API 解释\n\n### 初始化\n\n```python\nfrom net_inspect import NetInspect\n\nnet = NetInspect()\n```\n\n### 设置日志级别\n\n```python\nnet.verbose(1)\n```\n\n开启`verbose`表示为打开了`DEBUG`级别的日志, 默认为`INFO`。\n`verbose`级别可以通过`verbose()`方法设置，总共0~3\n- 0: 日志关闭\n- 1: 提供Output模块的日志和Parse模块的日志\n- 2: 追加提供Analysis模块的日志\n- 3: 追加提供Parse模块不支持命令的信息日志和命令为无效的信息\n\n### 设置插件\n\n一般来说，只需要设置`input_plugin`和`output_plugin`即可。\n\n```python\nnet.set_plugins(input_plugin=\'smartone\', output_plugin=\'device_warning_logging\')\n```\n可以使用字符串的简写，也可以自己继承插件类后重写`main()`方法，然后将类传递进来。\n\n### 执行\n\n提供输入的文件路径即可, 可以是文件或者目录。\n\n```python\nnet.run(\'log_files\')\n```\n\n如果output中需要提供参数，可以使用`output_plugin_params`参数，例如：\n\n```python\nnet.run(\'log_files\', output_plugin_params={\'company\': \'Company Name\'})\n```\n\n### 获取设备的基本信息\n\n已经提供了获取设备的基本信息的方法，可以通过`NetInspect.get_base_info()`方法获取。\n\n例如:\n```python\nnet.run(\'log_files\')\nall_info = net.get_base_info()\n\nfor info in all_info:\n    print(info.hostname)\n    print(info.sn)\n```\n\n#### 新增设备基本信息类\n\n```py\nfrom net_inspect import NetInspect, get_base_info, EachVendorDeviceInfo, BaseInfo, Device\n\n\nclass AppendClock(BaseInfo):\n    clock: str = \'\'  # 巡检时间\n\n\nclass EachVendorWithClock(EachVendorDeviceInfo):\n\n    base_info_class = AppendClock\n\n    def do_huawei_vrp_baseinfo_2(self, device: Device, info: AppendClock):\n        with device.search_cmd(\'display clock\') as cmd:\n            if cmd.parse_result:\n                row = cmd.parse_result[0]\n                info.clock = f\'{row["year"]}-{row["month"]}-{row["day"]} {row["time"]}\'\n\n    # def do_<other_vendor_platform>_baseinfo_<index>\n    #     ...\n\n\nnet = NetInspect()\nnet.set_input_plugin(\'smartone\')\nnet.run_input(\'地市巡检\')\n\nnet.set_input_plugin(\'console\')\nnet.run_input(\'cisco\')\n\nnet.run_parse()\nnet.run_analysis()\n\nwith open(\'base_info.csv\', \'w\') as f:\n    for device in net.cluster.devices:\n        info = get_base_info(device, EachVendorWithClock)  # type: AppendClock\n        row = [\n            info.hostname,\n            info.vendor_platform,\n            info.model,\n            info.ip,\n            info.version,\n            info.uptime,\n            info.cpu_usage,\n            info.memory_usage,\n            info.clock  # 这是新增的\n        ]\n        f.write(\' | \'.join(row) + "\\n")\n```\n\n\n## CLI 命令行操作\n\n![](resource/cli_1.png)\n\n```bash\nnet_inspect -i log_files\n```\n\n\n\n## 关于贡献\n\n分析插件还在持续开发中，`develop_script.py`脚本就是为高效开发提供的一个工具。\n\n开发一个分析插件的流程，以开发检查风扇状态的`fan_status`插件为例：\n\n1. 创建一个新的插件文件, 对应的文件初始状态会一并准备好\n\n```bash\npython ./develop_script.py -p fan_status -g\n```\n\n2. 在对应的文件中实现插件对每个厂商分析的函数\n\n```py\nclass AnalysisPluginWithFanStatus(AnalysisPluginAbc):\n    """\n    要求设备所有在位风扇模块运行在正常状态。\n    """\n    @analysis.vendor(vendor.H3C)\n    @analysis.template_key(\'hp_comware_display_fan.textfsm\', [\'slot\', \'id\', \'status\'])\n    def hp_comware(template: TemplateInfo, result: AnalysisResult):\n        """模块状态不为Normal的时候告警"""\n        for row in template[\'display fan\']:\n            if row[\'status\'].lower() != \'normal\':\n                result.add_warning(\n                    f\'Slot {row["slot"]} Fan {row["id"]} 状态异常\' if row[\'slot\'] else f\'Fan {row["id"]} 状态异常\')\n```\n\n其中`@analysis`是用来记录插件的分析类型的，`vendor`记录插件的厂商类型，`template_key`记录分析模块所需要的`textfsm`文件以及里面的哪些值。\n这些值会在参数`template: TemplateInfo`中给出。\n\n`result: AnalysisResult`用来记录分析结果。可以添加告警信息。\n\n分析方法为类方法，不需要`self`,不需要给出返回值。\n\n插件中的类注释和方法注释都会被记录下来，方便后续调用。\n\n3. 创建对应的测试文件\n\n当编写了对应的分析方法后，再次执行创建命令，工具会自动根据分析方法中需要的命令，生成对应的测试文件。\n\n测试文件路径为`tests/check_analysis_plugins/<plugin_name>/<funcation_name>.raw`\n\n```bash\npython ./development_script.py -p fan_status -f hp_comware -g\n```\n\n4. 在测试文件中添加测试用例\n5. 执行测试\n\n```bash\npython ./development_script.py -p fan_status -f hp_comware -t\n```\n\n6. 完成测试，确认测试结果为正常后，生成yml文件作为参考文件。\n\n```bash\npython ./development_script.py -p fan_status -f hp_comware -y\n```\n\n',
    'author': 'Elin',
    'author_email': '365433079@qq.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/Elinpf/net_inspect',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from __future__ import annotations

from typing import TYPE_CHECKING

from .. import vendor
from ..analysis_plugin import AnalysisPluginAbc, analysis

if TYPE_CHECKING:
    from ..analysis_plugin import TemplateInfo
    from ..domain import AnalysisResult


class AnalysisPluginWithMemoryStatus(AnalysisPluginAbc):
    """
    检查内存利用率不能过高。
    """

    @analysis.vendor(vendor.Huawei)
    @analysis.template_key('huawei_vrp_display_memory-usage.textfsm', ['MEMORY_USING_PERCENT'])
    def huawei_vrp(template_info: TemplateInfo, result: AnalysisResult):
        """
        内存利用率高于80%，则告警。
        """
        for row in template_info['display memory-usage']:
            check_memory_usage(float(row['MEMORY_USING_PERCENT']), result)

    @analysis.vendor(vendor.Cisco)
    @analysis.template_key('cisco_ios_show_processes_memory_sorted.textfsm', ['memory_total', 'memory_used'])
    @analysis.template_key('cisco_ios_show_processes_memory.textfsm', ['memory_total', 'memory_used'])
    def cisco_ios(template_info: TemplateInfo, result: AnalysisResult):
        """
        内存利用率高于80%，则告警。
        """
        for row in template_info['show processes memory sorted']:
            check_memory_usage(
                float(row['memory_used']) / float(row['memory_total']) * 100, result)

        for row in template_info['show processes memory']:
            check_memory_usage(
                float(row['memory_used']) / float(row['memory_total']) * 100, result)

    @analysis.vendor(vendor.Ruijie)
    @analysis.template_key('ruijie_os_show_memory.textfsm', ['system_memory_used_rate_precent'])
    def ruijie_os(template_info: TemplateInfo, result: AnalysisResult):
        """
        内存利用率高于80%，则告警。
        """
        for row in template_info['show memory']:
            check_memory_usage(
                float(row['system_memory_used_rate_precent']), result)

    @analysis.vendor(vendor.Maipu)
    @analysis.template_key('maipu_mypower_show_memory.textfsm', ['used_percent'])
    def maipu_mypower(template_info: TemplateInfo, result: AnalysisResult):
        """
        内存利用率高于80%，则告警。
        """
        for row in template_info['show memory']:
            check_memory_usage(float(row['used_percent']), result)

    @analysis.vendor(vendor.H3C)
    @analysis.template_key('hp_comware_display_memory.textfsm', ['used_rate'])
    def hp_comware(template_info: TemplateInfo, result: AnalysisResult):
        """
        内存利用率高于80%，则告警。
        """
        for row in template_info['display memory']:
            check_memory_usage(float(row['used_rate']), result)


def check_memory_usage(memory_usage: float, result: AnalysisResult):
    if memory_usage > 80:
        result.add_warning(f"内存利用率达到{round(memory_usage,1)}%")
